import pandas as pd
import numpy as np
from typing import Any, Dict, List, Callable
from copy import deepcopy

class ArrayWithMaps:
    """
    Array with maps corresponding to index and column locations.

    Keyword arguments
    -----------------
    array :
        data array
    rowsMap :
        dict of row indices from name -> int index
    colsMap :
        dict of column indices from name -> int index

    Attributes
    ----------
    array :
        data array
    rowsMap :
        dict of row indices from name -> int index
    colsMap :
        dict of column indices from name -> int index
    rowsMapReversed :
        dict of row indices from int -> name index
    colsMapReversed :
        dict of column indices from int -> name index

    Methods
    -------
    loc :
        Get value for provided named row and column.
    iloc :
        Get value for provided numbered row and column.
    ilocRow :
        Get value for provided numbered row and named column.
    getInd :
        Get the row index name from provided row number.
    getCol :
        Get the column name from provided column number.
    getRowNum :
        Get the row number from provided row name.
    getColNum :
        Get the column number from provided column name.
    getFullIndex :
        Get list of all row names.
    getFullCols :
        Get list of all column names.
    getFullRowNums :
        Get list of all row numbers.
    getFullColNums :
        Get list of all column numbers.
    """
    def __init__(self, array: np.ndarray, rowsMap: Dict, colsMap: Dict):
        self.array = array
        self.rowsMap = rowsMap
        self.colsMap = colsMap

        self.rowsMapReversed = {val: key for key, val in self.rowsMap.items()}
        self.colsMapReversed = {val: key for key, val in self.colsMap.items()}

    def loc(self, rows: Any, cols: str = None):
        rows = self.rowsMap.keys() if rows is None else rows
        cols = self.colsMap.keys() if cols is None else cols
        rows = [rows] if not hasattr(rows, '__iter__') or type(rows) == str else rows
        cols = [cols] if not hasattr(cols, '__iter__') or type(cols) == str else cols
        inds = [self.rowsMap[x] for x in rows]
        colVals = [self.colsMap[x] for x in cols]
        return np.take(np.take(self.array, inds, axis=0), colVals, axis=1)

    def getVal(self, row: Any, col: Any):
        return self.array[self.rowsMap[row], self.colsMap[col]]

    def getRow(self, row: Any):
        return self.array[self.rowsMap[row], :]

    def iloc(self, rowNums: int, colNums: int):
        rowNums = [rowNums] if not hasattr(rowNums, '__iter__') else rowNums
        colNums = [colNums] if not hasattr(colNums, '__iter__') else colNums
        return np.take(np.take(self.array, rowNums, axis=0), colNums, axis=1)

    def ilocRow(self, rowNum: int, col: str):
        return self.array[rowNum, self.colsMap[col]]

    def getInd(self, rowNum: int):
        return self.rowsMapReversed[rowNum]

    def getCol(self, colNum: int):
        return self.colsMapReversed[colNum]

    def getRowNum(self, row: Any):
        return self.rowsMap[row]

    def getColNum(self, col: str):
        return self.colsMap[col]

    def getFullIndex(self):
        return list(self.rowsMap.keys())

    def getFullCols(self):
        return list(self.colsMap.keys())

    def getFullRowNums(self):
        return list(self.rowsMap.values())

    def getFullColNums(self):
        return list(self.colsMap.values())

    def getFullColData(self, col: str):
        return self.array[:, self.colsMap[col]]

    def indexOnColLoc(self, colToIndexOn, indexVal, colToReturnFrom):
        return self.loc(row=list(self.getFullColData(colToIndexOn)).index(indexVal), col=colToReturnFrom)

    def filterIndex(self, val: Any, comparisonFn: Callable):
        filteredIndex = {x: self.rowsMap[x] for x in self.rowsMap.keys() if comparisonFn(x, val)}
        newArray = np.take(self.array, list(filteredIndex.values()), axis=0)
        newRowsMap = {newInd: newI for newInd, newI in zip(filteredIndex.keys(), range(len(filteredIndex.keys())))}
        newColsMap = dict(self.colsMap)
        newArrayMap = ArrayWithMaps(array=newArray, rowsMap=newRowsMap, colsMap=newColsMap)
        return newArrayMap

    def filterColumns(self, val: Any, comparisonFn: Callable):
        filteredColumns = {x: self.colsMap[x] for x in self.colsMap.keys() if comparisonFn(x, val)}
        newArray = np.take(self.array, list(filteredColumns.values()), axis=1)
        newColsMap = {newInd: newI for newInd, newI in zip(filteredColumns.keys(), range(len(filteredColumns.keys())))}
        newRowsMap = dict(self.rowsMap)
        newArrayMap = ArrayWithMaps(array=newArray, rowsMap=newRowsMap, colsMap=newColsMap)
        return newArrayMap

    def toDataframe(self):
        return pd.DataFrame(data=self.array, index=self.rowsMap.keys(), columns=self.colsMap.keys())

    def toSeries(self):
        return pd.Series(data=self.array[:, 0], index=self.rowsMap.keys())


import pandas as pd
import numpy as np
from datetime import timedelta
from datetime import date as dtDate
import importlib.util
import sys
import os
import re
from typing import Union
from copy import deepcopy
from functools import reduce
from operator import mul
from sklearn.model_selection import train_test_split
from win32com import client
from xlrd import xldate_as_datetime
from blp import blp

from .classes import ArrayWithMaps

from .variables import commodityInfo, weekdays, contractMonthInfo, variablesContractSizes
from .fastFunctions import *

def loadPricingDataframe(fileAddress):
    df = pd.read_csv(fileAddress, index_col=0)
    df.index = pd.to_datetime(df.index)
    if 'frontNotice' in df.columns:
        df['frontNotice'] = pd.to_datetime(df['frontNotice'])
    if 'lastViableDate' in df.columns:
        df['lastViableDate'] = pd.to_datetime(df['lastViableDate'])
    if 'fnd' in df.columns:
        df['fnd'] = pd.to_datetime(df['fnd'])
    if 'ltd' in df.columns:
        df['ltd'] = pd.to_datetime(df['ltd'])
    if 'lvd' in df.columns:
        df['lvd'] = pd.to_datetime(df['lvd'])
    return df

def today():
    return dt(dtDate.today())

def strContains(pattern, str):
    return re.search(pattern, str) is not None

def downwardDeviation(series):
    downwardReturns = series[series<-0.00000000001]
    if len(downwardReturns) == 0:
        return np.nan
    else:
        annualizedDD = downwardReturns.std(ddof=1) * (252 ** 0.5)
        ddVol = (2 ** 0.5) * annualizedDD
    return ddVol

def cagr(startVal, endVal, startDate, endDate):
    time = (endDate - startDate).days / 365

    return ((endVal / startVal) ** (1 / time)) - 1

def mean(lst):
    return sum(lst) / len(lst)

def parseDateToStrs(date):
    yearStr = str(date.year)
    monthStr = str(date.month) if date.month >= 10 else '0' + str(date.month)
    dayStr = str(date.day) if date.day >= 10 else '0' + str(date.day)

    return yearStr, monthStr, dayStr

def calculatePriorWeekday(date):
    if date.weekday() == 0: #if monday
        return date + timedelta(days=-3)
    elif date.weekday() == 6: #if sunday
        return date + timedelta(days=-2)
    else:
        return date + timedelta(days=-1)

def calculateNextWeekday(date):
    if date.weekday() == 4: #if friday
        return date + timedelta(days=3)
    elif date.weekday() == 5: #if saturday
        return date + timedelta(days=2)
    else:
        return date + timedelta(days=1)

def calculateClosestWeekday(date):
    if date.weekday() == 5: #if saturday
        return date + timedelta(days=-1)
    elif date.weekday() == 6: #if sunday
        return date + timedelta(days=1)
    else:
        return date

def calculatePriorWeekdayIfWeekend(date):
    if date.weekday() == 5 or date.weekday() == 6:
        return calculatePriorWeekday(date)
    else:
        return date

def dt(dateStr):
    return pd.to_datetime(dateStr)

def reverseDict(dictionary):
    return {value: key for key, value in dictionary.items()}

def importByFilepath(filepath, moduleName):
    cwd = os.getcwd()

    spec = importlib.util.spec_from_file_location(moduleName, filepath)
    outputModule = importlib.util.module_from_spec(spec)
    sys.modules[moduleName] = outputModule
    spec.loader.exec_module(outputModule)

    os.chdir(cwd)

    return outputModule

def getMonthCols(priceDf):
    monthCols = [x for x in priceDf.columns if re.search('Month', x) is not None]
    return priceDf.loc[:, monthCols]

def getPriceCols(priceDf):
    priceCols = [x for x in priceDf.columns if re.search('Price', x) is not None]
    return priceDf.loc[:, priceCols]

def pandasToNumpy(pandasObj: Union[pd.DataFrame,pd.Series]):
    """
    Generate a numpy array from pandasObj with index and col maps.
    """
    npArray = pandasObj.to_numpy()
    if type(pandasObj) == pd.Series:
        colsMap = {}
    else:
        colsMap = {col: i for col, i in zip(pandasObj.columns, range(len(pandasObj.columns)))}
    rowsMap = {ind: i for ind, i in zip(pandasObj.index, range(len(pandasObj.index)))}
    return npArray, rowsMap, colsMap

def initializeArrayFromDataframe(dataframe: pd.DataFrame):
    """Generate ArrayWithMaps object from dataframe."""
    array, rowsMap, colsMap = pandasToNumpy(dataframe)
    arrayInstance = ArrayWithMaps(array, rowsMap, colsMap)
    return arrayInstance

def initializeArrayFromFile(fileAddress: str):
    """Generate ArrayWithMaps object from csv file."""
    dataframe = loadPricingDataframe(fileAddress)
    return initializeArrayFromDataframe(dataframe)

def initializeArrayFromDict(dict: dict, indexKey: any = None, maintainOriginalDict: bool = True):
    """
    Generate ArrayWithMaps object from dict.

    Will use number indexing if none of the keys specified as index.
    maintainOriginalDict set to False will give a performance boost but will destroy the input dictionary.
    """
    if maintainOriginalDict:
        dict = deepcopy(dict)
    if indexKey is not None:
        index = dict.pop(indexKey)
    else:
        index = list(range(len(dict)))
    cols = list(dict.keys())
    data = list(dict.values())
    arrayObj = ArrayWithMaps(
        array=np.array(data).T,
        rowsMap={ind: i for ind, i in zip(index, range(len(index)))},
        colsMap={col: i for col, i in zip(cols, range(len(cols)))}
    )
    return arrayObj

def weightedAverage(values, weights):
    return np.average(values, weights=weights)

def weightedStDev(values, weights):
    wAvg = weightedAverage(values, weights)
    sumProduct = sum([((x - wAvg) ** 2) * y for x, y in zip(values, weights)])
    adjSumWeights = sum(weights)# - 1
    return (sumProduct / adjSumWeights) ** 0.5

def dayOfWeekNP(dates):
    return (dates.astype('datetime64[D]').view('int64') - 4) % 7

def product(lst):
    return reduce(mul, lst, 1)

def arrayTrainTestSplit(array: ArrayWithMaps, testSize: float, setIndexToOriginal: bool = False):
    df = array.toDataframe()
    train, test = train_test_split(df, test_size=testSize)
    train, test = train.sort_index(), test.sort_index()
    if setIndexToOriginal:
        train.index = df.index[:train.shape[0]]
        test.index = df.index[:test.shape[0]]
    trainArray = initializeArrayFromDataframe(train)
    testArray = initializeArrayFromDataframe(test)
    return trainArray, testArray

def calculateReturnsOfSeries(priceSeries: pd.Series):
    return priceSeries / priceSeries.shift(1) - 1

def writeDfToExcel(df, excelFileLocation, sheetName, writeIndex=False, useOpenPyXlAsEngine=True):
    """Use xlsxwriter only if you MUST format dates and do not care if the write deletes all other sheets."""
    if useOpenPyXlAsEngine: #!should use openpyxl if not trying to format dates
        with pd.ExcelWriter(excelFileLocation, engine='openpyxl', mode='a', if_sheet_exists='replace') as writer:
            df.to_excel(writer, sheet_name=sheetName, index=writeIndex)
    else:
        with pd.ExcelWriter(excelFileLocation, engine='xlsxwriter', datetime_format='m/d/yyyy', date_format='m/d/yyyy') as writer:
            df.to_excel(writer, sheet_name=sheetName, index=writeIndex)

def sendEmail(
    to,
    subject,
    body=None,
    attachmentAddresses=[]
):
    outlook = client.Dispatch("Outlook.Application")

    mail = outlook.CreateItem(0)
    mail.To = to
    mail.Subject = subject
    if body is not None:
        mail.Body = body

    if attachmentAddresses != []:
        for attachmentAddress in attachmentAddresses:
            mail.Attachments.Add(attachmentAddress)

    mail.Send()

def makeAppointment(
    subject: str,
    startDate: pd.Timestamp,
    startHour24Fmt: int,
    startMinute: int,
    reminderMinutesBefore=5,
    body: str = None,
    location: str = None
):
    outlook = client.Dispatch("Outlook.Application")

    appointment = outlook.CreateItem(1)

    startStr = f"{startDate.strftime('%m/%d/%Y')} {startHour24Fmt}:{str(startMinute) if startMinute >= 10 else '0' + str(startMinute)}:00"

    appointment.subject = subject
    appointment.start = startStr
    appointment.duration = 0
    appointment.reminderset=True
    appointment.reminderminutesbeforestart = reminderMinutesBefore

    if location is not None:
        appointment.location = location
    if body is not None:
        appointment.body = body

    appointment.save()

def calculateRsi(priceSeries, periods=14):
    priceChanges = priceSeries.diff()

    upPriceChanges = priceChanges.clip(lower=0)
    downPriceChanges = - priceChanges.clip(upper=0) #these are positive

    upPriceChangesWeightedAverages = upPriceChanges.ewm(com=periods-1, min_periods=periods).mean()
    downPriceChangesWeightedAverages = downPriceChanges.ewm(com=periods-1, min_periods=periods).mean()

    upDownRatio = upPriceChangesWeightedAverages / downPriceChangesWeightedAverages

    rsi = 100 - (100 / (1 + upDownRatio))

    return rsi

def generateCommodityInfoConverter(fromKey, toKey):
    """
    Keys: shortName, oldName, formalName, bloombergSymbol, yellowKey, systemTicker, sector, drwSymbol, currency, varName, contractSize, bloombergDollarDivisor, settlementTime
    """
    return {singleCommodityInfo[fromKey]: singleCommodityInfo[toKey] for singleCommodityInfo in commodityInfo.values()}

def generateContractMonthConverter(fromKey, toKey):
    """
    Keys: int, bloombergCode, abbreviation, fullName
    """
    return {singleContractMonthInfo[fromKey]: singleContractMonthInfo[toKey] for singleContractMonthInfo in contractMonthInfo.values()}

def calculateNWeekdaysShift(date, n):
    isWeekend = date.weekday() == 5 or date.weekday() == 6
    if isWeekend and n < 0:
        date = calculatePriorWeekday(date)
    elif isWeekend and n > 0:
        date = calculateNextWeekday(date)

    dateInd = np.where(weekdays == date)[0][0]

    return dt(weekdays[dateInd + n])

def calculateWeekdaysBetween(startDate, endDate):
    subset = weekdays[(weekdays >= startDate) & (weekdays <= endDate)]
    return len(subset)

def calculateContractSize(bloombergTicker, monthBloombergCode: str=None, year: int=None):
    if commodityInfo[bloombergTicker.lower()]['variableContractSize']:
        if monthBloombergCode is None or year is None:
            raise IndexError("Enter a month for variable contract commodity.")
        vcsDf = variablesContractSizes[bloombergTicker.lower()]
        contractSize = vcsDf.loc[(vcsDf['monthCode'] == monthBloombergCode) & (vcsDf['year'] == year), 'contSize'].values[0]
    else:
        contractSize = commodityInfo[bloombergTicker.lower()]['contractSize']

    return contractSize

def camelToSnake(camelCaseString):
   snakeCaseString = re.sub('([A-Z])', r'_\1', camelCaseString).lower()
   return snakeCaseString

def snakeToCamel(snakeCaseStr):
   camelCaseString = re.sub(r'_([a-zA-Z])', lambda match: match.group(1).upper(), snakeCaseStr)
   return camelCaseString

def loadExcelForCheck(excelFilePath, sheetName='Sheet1', columnNames=None, excelColumns=None, skiprows=0, dtypes=None):
    usingXlsb = re.search('\.xlsb', excelFilePath) is not None
    engine = 'pyxlsb' if usingXlsb else 'openpyxl'

    if columnNames is None:
        df = pd.read_excel(
            io=excelFilePath,
            sheet_name=sheetName,
            header=0,
            index_col=0,
            skiprows=skiprows,
            engine=engine,
        )
    else:
        df = pd.read_excel(
            io=excelFilePath,
            sheet_name=sheetName,
            header=None,
            names=columnNames,
            index_col=0,
            usecols=excelColumns,
            skiprows=skiprows,
            engine=engine,
        )
    df = df[~df.eq('0x2a').any(1)] #remove any rows with blanks
    df = df[~df.eq('0x7').any(1)] #remove any rows with blanks
    df = df[~df.eq('0x17').any(1)] #remove any rows with blanks

    if usingXlsb:
        df.index = [xldate_as_datetime(x, datemode=0) for x in df.index]
        for col in [key for key, value in dtypes.items() if re.search('date', value)]:
            df[col] = df[col].fillna(0)
            df[col] = [xldate_as_datetime(x, datemode=0) for x in df[col]]
            df[col] = df[col].replace('12/31/1899', np.nan)

    if dtypes is not None:
        df = df.astype(dtypes)

    df.index = pd.to_datetime(df.index)
    return df

def compareTwoDfs(df1, df2, threshold=0.00001):
    resDict = {}

    df1ColsSet = set(df1.columns)
    sharedColumns = [x for x in df2.columns if x in df1ColsSet]

    #sharedColumns = list(set(df1.columns).intersection(set(df2.columns)))

    df1 = df1[sharedColumns]
    df2 = df2[sharedColumns]
    df1 = df1.loc[(df1.index >= df2.index.min()) & (df1.index <= df2.index.max())]
    df2 = df2.loc[(df2.index >= df1.index.min()) & (df2.index <= df1.index.max())]

    for col in sharedColumns:
        if df1[col].dtype == 'bool':
            df1[col] = deepcopy(df1[col]).astype('int')
        if df2[col].dtype == 'bool':
            df2[col] = deepcopy(df2[col]).astype('int')
        if df1[col].dtype == 'O':
            df1[col] = [hash(x) for x in df1[col]]
        if df2[col].dtype == 'O':
            df2[col] = [hash(x) for x in df2[col]]

        diffed = abs(df1[col].fillna(999999999) - df2[col].fillna(999999999))
        diffed[diffed > threshold] = 1
        diffed[diffed <= threshold] = 0
        firstError = str(diffed.index[diffed == 1][0].date()) if diffed.sum() != 0 else 'N/A'
        resDict[col] = [diffed.sum(), firstError]

    if df1.empty or df2.empty:
        for key in resDict.keys():
            resDict[key] = [999999, dt('1/1/1900')]

    return resDict

def printCheckResults(resDict, printFile=sys.stdout):
    earliestError = dt('1/1/2100')
    earliestErrorItem = ''
    totalErrors = 0

    for key, value in resDict.items():
        totalErrors += value[0]
        print(f"{key}: {int(value[0])} ({value[1]})", file=printFile)
        if value[1] != 'N/A':
            if dt(value[1]) < dt(earliestError):
                earliestError = value[1]
                earliestErrorItem = key

    print(file=printFile)

    if earliestErrorItem != '':
        print(f"Earliest error on {earliestError} for {earliestErrorItem}", file=printFile)

    print(file=printFile)

def findActiveFile(folder, fileGeneralName, fileExtension, use_vFormat=True):
    """
    Allows finding the address of the most recent version of an evolving file with the format e.g. "filename_v01.txt" (if use_vFormat is True) or "filename1.txt" (if use_vFormat is False).

    folder: the folder where the evolving file lives
    fileGeneralName: the part of the filename before the "_v01" in the example above if using _v format, or before the "1" if not using _v format
    fileExtension: the file type excluding the period
    use_vFormat: whether to assume the filename format includes "_v" at the end

    Returns the whole file address, including the folder.
    """
    pattern = f"^{fileGeneralName}_v(\d+).{fileExtension}$" if use_vFormat else f"^{fileGeneralName}(\d+).{fileExtension}$"
    excelFilesAndVersionNums = {int(re.search(pattern, x).group(1)): x for x in os.listdir(folder) if re.search(pattern, x)}
    latestFile = excelFilesAndVersionNums[max(excelFilesAndVersionNums.keys())]

    return f"{folder}/{latestFile}"

def dataframeToListOfLists(dataframe: pd.DataFrame, includeColumns=True, includeIndex=True, colsWithDates=[], dateIndex=False):
    dataframe = deepcopy(dataframe)
    if dateIndex:
        dataframe.index = dataframe.index.strftime('%m/%d/%Y')
    for colWithDate in colsWithDates:
        dataframe[colWithDate] = dataframe[colWithDate].dt.strftime('%m/%d/%Y')
    lol = dataframe.values.tolist()
    if includeIndex:
        for i, l in zip(range(len(lol)), lol):
            l.insert(0, dataframe.index[i])
    if includeColumns:
        if includeIndex:
            lol.insert(0, ['Index'] + list(dataframe.columns))
        else:
            lol.insert(0, list(dataframe.columns))

    return lol

def listOfListsToDataframe(listOfLists, includeColumns=True, indexLabel=None, colsWithDates=[], dateIndex=False):
    dataframe = pd.DataFrame(listOfLists)
    if includeColumns:
        dataframe.columns = dataframe.loc[0]
        dataframe = dataframe.iloc[1:]
    if indexLabel is not None:
        dataframe = dataframe.set_index(indexLabel)
    if dateIndex:
        dataframe.index = dt(dataframe.index)
    for colWithDate in colsWithDates:
        dataframe[colWithDate] = dt(dataframe[colWithDate])

    return dataframe

def largestPeriodFinder(series: pd.Series, numPeriods: int, aggregatorFn, numberToReturn: int=10):
    aggregations = {
        'startIndex': [],
        'endIndex': [],
        'startInt': [],
        'endInt': [],
        'aggregation': [],
    }
    for i in range(len(series) - numPeriods):
        startInt = i
        endInt = i + numPeriods
        startIndex = series.index[i]
        endIndex = series.index[i + numPeriods]
        aggregated = aggregatorFn(series[i:i + numPeriods])

        aggregations['startIndex'].append(startIndex)
        aggregations['endIndex'].append(endIndex)
        aggregations['startInt'].append(startInt)
        aggregations['endInt'].append(endInt)
        aggregations['aggregation'].append(aggregated)

    df = pd.DataFrame(aggregations)

    bests = []
    for _ in range(numberToReturn):
        best = df.loc[df['aggregation'] == df['aggregation'].min()]
        bests.append(best)
        df = df.loc[~((df['startInt'] >= best.loc[best.index[0], 'startInt']) & (df['startInt'] <= best.loc[best.index[0], 'endInt'])) & ~((df['endInt'] >= best.loc[best.index[0], 'startInt']) & (df['endInt'] <= best.loc[best.index[0], 'endInt']))]

    bestDf = pd.concat(bests)

    return bestDf

def addNextPeriodResult(periodsDf, series, days):
    results = []
    for i in periodsDf.index:
        startDate = calculateNWeekdaysShift(periodsDf.loc[i, 'endIndex'], 1)
        endDate = calculateNWeekdaysShift(periodsDf.loc[i, 'endIndex'], days)
        #print(startDate, endDate)
        result = series.loc[(series.index >= startDate) & (series.index <= endDate)].sum()
        results.append(result)
    periodsDf[f'next{days}Result'] = results
    return periodsDf

def pullBloombergDataFrame(
    securities,
    fields,
    startDate,
    endDate,
    includeAllWeekdays=True,
    connection=None
):
    if connection is None:
        connection = blp.BlpQuery().start()

    nonTradingDayFillOption = 'NON_TRADING_WEEKDAYS' if includeAllWeekdays else 'ACTIVE_DAYS_ONLY'

    raw = connection.bdh(
        securities=securities,
        fields=fields,
        start_date=startDate.strftime('%Y%m%d'),
        end_date=endDate.strftime('%Y%m%d'),
        options=[('nonTradingDayFillOption', nonTradingDayFillOption)]
    )
    df = raw.pivot(columns=['security'], index='date')
    return df


import argparse

def argParseDriver(
    keywords,
    required=[],
    defaults={}
):
    parser = argparse.ArgumentParser()
    for keyword in keywords:
        keyword = '--' + keyword.strip('-')
        parser.add_argument(
            keyword,
            required=True if keyword.strip('-') in required else None,
            default=defaults[keyword.strip('-')] if keyword.strip('-') in defaults.keys() else None
        )
    return parser.parse_args()


import pandas as pd

def reverseDict(dictionary):
    return {value: key for key, value in dictionary.items()}

contractMonthInfo = {
    1: {
        'int': 1,
        'bloombergCode': 'F',
        'abbreviation': 'JAN',
        'fullName': 'January'
    },
    2: {
        'int': 2,
        'bloombergCode': 'G',
        'abbreviation': 'FEB',
        'fullName': 'February'
    },
    3: {
        'int': 3,
        'bloombergCode': 'H',
        'abbreviation': 'MAR',
        'fullName': 'March'
    },
    4: {
        'int': 4,
        'bloombergCode': 'J',
        'abbreviation': 'APR',
        'fullName': 'April'
    },
    5: {
        'int': 5,
        'bloombergCode': 'K',
        'abbreviation': 'MAY',
        'fullName': 'May'
    },
    6: {
        'int': 6,
        'bloombergCode': 'M',
        'abbreviation': 'JUN',
        'fullName': 'June'
    },
    7: {
        'int': 7,
        'bloombergCode': 'N',
        'abbreviation': 'JUL',
        'fullName': 'July'
    },
    8: {
        'int': 8,
        'bloombergCode': 'Q',
        'abbreviation': 'AUG',
        'fullName': 'August'
    },
    9: {
        'int': 9,
        'bloombergCode': 'U',
        'abbreviation': 'SEP',
        'fullName': 'September'
    },
    10: {
        'int': 10,
        'bloombergCode': 'V',
        'abbreviation': 'OCT',
        'fullName': 'October'
    },
    11: {
        'int': 11,
        'bloombergCode': 'X',
        'abbreviation': 'NOV',
        'fullName': 'November'
    },
    12: {
        'int': 12,
        'bloombergCode': 'Z',
        'abbreviation': 'DEC',
        'fullName': 'December'
    },
}

contractMonthCodeToInt = {
    'F': 1,
    'G': 2,
    'H': 3,
    'J': 4,
    'K': 5,
    'M': 6,
    'N': 7,
    'Q': 8,
    'U': 9,
    'V': 10,
    'X': 11,
    'Z': 12
}
contractMonthIntToCode = reverseDict(contractMonthCodeToInt)

contractMonthCodeToStr = {
    'F': 'JAN',
    'G': 'FEB',
    'H': 'MAR',
    'J': 'APR',
    'K': 'MAY',
    'M': 'JUN',
    'N': 'JUL',
    'Q': 'AUG',
    'U': 'SEP',
    'V': 'OCT',
    'X': 'NOV',
    'Z': 'DEC'
}
contractMonthStrToCode = reverseDict(contractMonthCodeToStr)

contractTermIntToStr = {
    0: 'front',
    1: 'second',
    2: 'third',
    3: 'fourth',
    4: 'fifth',
    5: 'sixth',
    6: 'seventh',
    7: 'eighth',
    8: 'ninth',
    9: 'tenth',
    10: 'eleventh',
    11: 'twelfth'
}
contractTermStrToInt = reverseDict(contractTermIntToStr)

colorDict = {
    'black': '#17202A',
    'darkGrey': '#4D5656',
    'medGrey': '#B3B6B7',
    'lightGrey': '#F0F3F4',
    'darkBrown': '#6E2C00',
    'medBrown': '#BA4A00',
    'burntOrange': '#CA6F1E',
    'lightOrange': '#EB984E',
    'burntOchre': '#B7950B',
    'gold': '#F1C40F',
    'softYellow': '#F7DC6F',
    'darkGreen': '#196F3D',
    'medGreen': '#27AE60',
    'lightGreen': '#7DCEA0',
    'darkBlue': '#1A5276',
    'medBlue': '#2980B9',
    'lightBlue': '#A9CCE3',
    'darkPurple': '#4A235A',
    'medPurple': '#A569BD',
    'lightPurple': '#E8DAEF',
    'darkRed': '#922B21',
    'medRed': '#CD6155',
    'pink': '#F2D7D5'
}

commodityInfo = {
    'cl': {
        'shortName': 'cl',
        'oldName': 'crude',
        'formalName': 'WTI',
        'bloombergSymbol': 'CL',
        'yellowKey': 'Comdty',
        'systemTicker': 'cl',
        'sector': 'Energy',
        'drwSymbol': 'CL',
        'currency': 'USD',
        'varName': 'WTI',
        'variableContractSize': False,
        'contractSize': 1_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'cl_alt': {
        'shortName': 'cl',
        'oldName': 'crude',
        'formalName': 'WTI',
        'bloombergSymbol': 'CL',
        'yellowKey': 'Comdty',
        'systemTicker': 'cl',
        'sector': 'Energy',
        'drwSymbol': 'WBS',
        'currency': 'USD',
        'varName': 'WTI',
        'variableContractSize': False,
        'contractSize': 1_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'co': {
        'shortName': 'co',
        'oldName': 'brent',
        'formalName': 'Brent',
        'bloombergSymbol': 'CO',
        'yellowKey': 'Comdty',
        'systemTicker': 'co',
        'sector': 'Energy',
        'drwSymbol': 'BC',
        'currency': 'USD',
        'varName': 'Brent',
        'variableContractSize': False,
        'contractSize': 1_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'co_alt': {
        'shortName': 'co',
        'oldName': 'brent',
        'formalName': 'Brent',
        'bloombergSymbol': 'CO',
        'yellowKey': 'Comdty',
        'systemTicker': 'co',
        'sector': 'Energy',
        'drwSymbol': 'BZ',
        'currency': 'USD',
        'varName': 'Brent',
        'variableContractSize': False,
        'contractSize': 1_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'ho': {
        'shortName': 'ho',
        'oldName': 'diesel',
        'formalName': 'Diesel',
        'bloombergSymbol': 'HO',
        'yellowKey': 'Comdty',
        'systemTicker': 'ho',
        'sector': 'Energy',
        'drwSymbol': 'HO',
        'currency': 'USD',
        'varName': 'Heating Oil',
        'variableContractSize': False,
        'contractSize': 42_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:30',
    },
    'xb': {
        'shortName': 'xb',
        'oldName': 'gasoline',
        'formalName': 'Gasoline',
        'bloombergSymbol': 'XB',
        'yellowKey': 'Comdty',
        'systemTicker': 'xb',
        'sector': 'Energy',
        'drwSymbol': 'RB',
        'currency': 'USD',
        'varName': 'RBOB',
        'variableContractSize': False,
        'contractSize': 42_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:30',
    },
    'ng': {
        'shortName': 'ng',
        'oldName': 'natgas',
        'formalName': 'Natural Gas',
        'bloombergSymbol': 'NG',
        'yellowKey': 'Comdty',
        'systemTicker': 'ng',
        'sector': 'Energy',
        'drwSymbol': 'NG',
        'currency': 'USD',
        'varName': 'Henry Hub',
        'variableContractSize': False,
        'contractSize': 10_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'tzt': {
        'shortName': 'tzt',
        'oldName': 'ttf',
        'formalName': 'TTF',
        'bloombergSymbol': 'TZT',
        'yellowKey': 'Comdty',
        'systemTicker': 'ttf',
        'sector': 'Energy',
        'drwSymbol': 'TTF',
        'currency': 'EUR',
        'varName': 'Dutch',
        'variableContractSize': True,
        'contractSize': 'C:/Users/jkarisch/Projects/common/data/ttfContractSizes.csv',
        'bloombergDollarDivisor': 1,
        'settlementTime': '11:15',
    },
    'bap': {
        'shortName': 'bap',
        'oldName': 'propane',
        'formalName': 'Propane',
        'bloombergSymbol': 'BAP',
        'yellowKey': 'Comdty',
        'systemTicker': 'bap',
        'sector': 'Energy',
        'drwSymbol': 'B0',
        'currency': 'USD',
        'varName': 'Propane',
        'variableContractSize': False,
        'contractSize': 42_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'zls': {
        'shortName': 'zls',
        'oldName': 'pjm',
        'formalName': 'PJM',
        'bloombergSymbol': 'ZLS',
        'yellowKey': 'Comdty',
        'systemTicker': 'pjm',
        'sector': 'Energy',
        'drwSymbol': 'PMI',
        'currency': 'USD',
        'varName': 'PJM',
        'variableContractSize': True,
        'contractSize': 'C:/Users/jkarisch/Projects/common/data/pjmContractSizes.csv',
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
    'gc': {
        'shortName': 'gc',
        'oldName': 'gold',
        'formalName': 'Gold',
        'bloombergSymbol': 'GC',
        'yellowKey': 'Comdty',
        'systemTicker': 'gc',
        'sector': 'Metals',
        'drwSymbol': 'GC',
        'currency': 'USD',
        'varName': 'Gold',
        'variableContractSize': False,
        'contractSize': 100,
        'bloombergDollarDivisor': 1,
        'settlementTime': '13:30',
    },
    'si': {
        'shortName': 'si',
        'oldName': 'silver',
        'formalName': 'Silver',
        'bloombergSymbol': 'SI',
        'yellowKey': 'Comdty',
        'systemTicker': 'si',
        'sector': 'Metals',
        'drwSymbol': 'SI',
        'currency': 'USD',
        'varName': 'Silver',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '13:25',
    },
    'pl': {
        'shortName': 'pl',
        'oldName': 'platinum',
        'formalName': 'Platinum',
        'bloombergSymbol': 'PL',
        'yellowKey': 'Comdty',
        'systemTicker': 'pl',
        'sector': 'Metals',
        'drwSymbol': 'PL',
        'currency': 'USD',
        'varName': 'Platinum',
        'variableContractSize': False,
        'contractSize': 50,
        'bloombergDollarDivisor': 1,
        'settlementTime': '13:05',
    },
    'pa': {
        'shortName': 'pa',
        'oldName': 'palladium',
        'formalName': 'Palladium',
        'bloombergSymbol': 'PA',
        'yellowKey': 'Comdty',
        'systemTicker': 'pa',
        'sector': 'Metals',
        'drwSymbol': 'PA',
        'currency': 'USD',
        'varName': 'Palladium',
        'variableContractSize': False,
        'contractSize': 100,
        'bloombergDollarDivisor': 1,
        'settlementTime': '13:00',
    },
    'hg': {
        'shortName': 'hg',
        'oldName': 'copper',
        'formalName': 'Copper',
        'bloombergSymbol': 'HG',
        'yellowKey': 'Comdty',
        'systemTicker': 'hg',
        'sector': 'Metals',
        'drwSymbol': 'CU',
        'currency': 'USD',
        'varName': 'Copper',
        'variableContractSize': False,
        'contractSize': 25_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '13:00',
    },
    'la': {
        'shortName': 'la',
        'oldName': 'aluminum',
        'formalName': 'Aluminum',
        'bloombergSymbol': 'LA',
        'yellowKey': 'Comdty',
        'systemTicker': 'la',
        'sector': 'Metals',
        'drwSymbol': 'LMEAHD',
        'currency': 'USD',
        'varName': 'Aluminum',
        'variableContractSize': False,
        'contractSize': 25,
        'bloombergDollarDivisor': 1,
        'settlementTime': '11:30',
    },
    'ln': {
        'shortName': 'ln',
        'oldName': 'nickel',
        'formalName': 'Nickel',
        'bloombergSymbol': 'LN',
        'yellowKey': 'Comdty',
        'systemTicker': 'ln',
        'sector': 'Metals',
        'drwSymbol': 'LMENID',
        'currency': 'USD',
        'varName': 'Nickel',
        'variableContractSize': False,
        'contractSize': 6,
        'bloombergDollarDivisor': 1,
        'settlementTime': '11:20',
    },
    'sco': {
        'shortName': 'sco',
        'oldName': 'iron',
        'formalName': 'Iron Ore',
        'bloombergSymbol': 'SCO',
        'yellowKey': 'Comdty',
        'systemTicker': 'sco',
        'sector': 'Metals',
        'drwSymbol': 'SGFEF',
        'currency': 'USD',
        'varName': 'Iron',
        'variableContractSize': False,
        'contractSize': 100,
        'bloombergDollarDivisor': 1,
        'settlementTime': '04:30',
    },
    'lh': {
        'shortName': 'lh',
        'oldName': 'leanhogs',
        'formalName': 'Lean Hogs',
        'bloombergSymbol': 'LH',
        'yellowKey': 'Comdty',
        'systemTicker': 'lh',
        'sector': 'Livestock',
        'drwSymbol': '54',
        'currency': 'USD',
        'varName': 'Lean Hogs',
        'variableContractSize': False,
        'contractSize': 40_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:00',
    },
    'fc': {
        'shortName': 'fc',
        'oldName': 'feeder',
        'formalName': 'Feeder Cattle',
        'bloombergSymbol': 'FC',
        'yellowKey': 'Comdty',
        'systemTicker': 'fc',
        'sector': 'Livestock',
        'drwSymbol': 'FC',
        'currency': 'USD',
        'varName': 'Feeder Cattle',
        'variableContractSize': False,
        'contractSize': 50_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:00',
    },
    'sm': {
        'shortName': 'sm',
        'oldName': 'soymeal',
        'formalName': 'Soybean Meal',
        'bloombergSymbol': 'SM',
        'yellowKey': 'Comdty',
        'systemTicker': 'sm',
        'sector': 'Grains',
        'drwSymbol': 'SM',
        'currency': 'USD',
        'varName': 'Soybean Meal',
        'variableContractSize': False,
        'contractSize': 100,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:15',
    },
    'kw': {
        'shortName': 'kw',
        'oldName': 'hrw',
        'formalName': 'KC Wheat',
        'bloombergSymbol': 'KW',
        'yellowKey': 'Comdty',
        'systemTicker': 'kw',
        'sector': 'Grains',
        'drwSymbol': 'KCB',
        'currency': 'USD',
        'varName': 'Wheat',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    'w ': {
        'shortName': 'w ',
        'oldName': 'srw',
        'formalName': 'Chicago Wheat',
        'bloombergSymbol': 'W ',
        'yellowKey': 'Comdty',
        'systemTicker': 'w',
        'sector': 'Grains',
        'drwSymbol': 'W',
        'currency': 'USD',
        'varName': 'Wheat',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    'c ': {
        'shortName': 'c ',
        'oldName': 'corn',
        'formalName': 'Corn',
        'bloombergSymbol': 'C ',
        'yellowKey': 'Comdty',
        'systemTicker': 'c',
        'sector': 'Grains',
        'drwSymbol': 'C',
        'currency': 'USD',
        'varName': 'Corn',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    's ': {
        'shortName': 's ',
        'oldName': 'soybean',
        'formalName': 'Soybeans',
        'bloombergSymbol': 'S ',
        'yellowKey': 'Comdty',
        'systemTicker': 's',
        'sector': 'Grains',
        'drwSymbol': 'S',
        'currency': 'USD',
        'varName': 'Soybeans',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    'sb': {
        'shortName': 'sb',
        'oldName': 'sugar',
        'formalName': 'Sugar',
        'bloombergSymbol': 'SB',
        'yellowKey': 'Comdty',
        'systemTicker': 'sb',
        'sector': 'Softs',
        'drwSymbol': 'S11',
        'currency': 'USD',
        'varName': 'Sugar',
        'variableContractSize': False,
        'contractSize': 112_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '12:55',
    },
    'ct': {
        'shortName': 'ct',
        'oldName': 'cotton',
        'formalName': 'Cotton',
        'bloombergSymbol': 'CT',
        'yellowKey': 'Comdty',
        'systemTicker': 'ct',
        'sector': 'Softs',
        'drwSymbol': 'CT',
        'currency': 'USD',
        'varName': 'Cotton',
        'variableContractSize': False,
        'contractSize': 50_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    'kc': {
        'shortName': 'kc',
        'oldName': 'coffee',
        'formalName': 'Coffee',
        'bloombergSymbol': 'KC',
        'yellowKey': 'Comdty',
        'systemTicker': 'kc',
        'sector': 'Softs',
        'drwSymbol': 'CF',
        'currency': 'USD',
        'varName': 'Coffee',
        'variableContractSize': False,
        'contractSize': 37_500,
        'bloombergDollarDivisor': 100,
        'settlementTime': '12:25',
    },
    'ko': {
        'shortName': 'ko',
        'oldName': 'palmoil',
        'formalName': 'Palm Oil',
        'bloombergSymbol': 'KO',
        'yellowKey': 'Comdty',
        'systemTicker': 'ko',
        'sector': 'Softs',
        'drwSymbol': 'FCPO',
        'currency': 'MYR',
        'varName': 'Palm Oil',
        'variableContractSize': False,
        'contractSize': 25,
        'bloombergDollarDivisor': 1,
        'settlementTime': '05:00',
    },
    'bo': {
        'shortName': 'bo',
        'oldName': 'beanoil',
        'formalName': 'Bean Oil',
        'bloombergSymbol': 'BO',
        'yellowKey': 'Comdty',
        'systemTicker': 'bo',
        'sector': 'Softs',
        'drwSymbol': 'BO',
        'currency': 'USD',
        'varName': 'Soybean Oil',
        'variableContractSize': False,
        'contractSize': 60_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    'rs': {
        'shortName': 'rs',
        'oldName': 'canola',
        'formalName': 'Canola',
        'bloombergSymbol': 'RS',
        'yellowKey': 'Comdty',
        'systemTicker': 'rs',
        'sector': 'Softs',
        'drwSymbol': 'W8',
        'currency': 'CAD',
        'varName': 'Canola',
        'variableContractSize': False,
        'contractSize': 20,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:15',
    },
    'ec': {
        'shortName': 'ec',
        'oldName': 'euro',
        'formalName': 'EUR',
        'bloombergSymbol': 'EC',
        'yellowKey': 'Curncy',
        'systemTicker': 'eur',
        'sector': 'FX',
        'drwSymbol': 'EC',
        'currency': 'USD',
        'varName': 'EUR-USD',
        'variableContractSize': False,
        'contractSize': 100_000,
        'bloombergDollarDivisor': 1,
        'settlementTime': '15:00',
    },
    'ad': {
        'shortName': 'ad',
        'oldName': 'aus',
        'formalName': 'AUD',
        'bloombergSymbol': 'AD',
        'yellowKey': 'Curncy',
        'systemTicker': 'aud',
        'sector': 'FX',
        'drwSymbol': 'AD',
        'currency': 'USD',
        'varName': 'AUD-USD',
        'variableContractSize': False,
        'contractSize': 100_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '15:00',
    },
    'jy': {
        'shortName': 'jy',
        'oldName': 'yen',
        'formalName': 'JPY',
        'bloombergSymbol': 'JY',
        'yellowKey': 'Curncy',
        'systemTicker': 'jpy',
        'sector': 'FX',
        'drwSymbol': 'JY',
        'currency': 'USD',
        'varName': 'USD-JPY',
        'variableContractSize': False,
        'contractSize': 12_500_000,
        'bloombergDollarDivisor': 10_000,
        'settlementTime': '15:00',
    },
    'br': {
        'shortName': 'br',
        'oldName': 'real',
        'formalName': 'BRL',
        'bloombergSymbol': 'BR',
        'yellowKey': 'Curncy',
        'systemTicker': 'brl',
        'sector': 'FX',
        'drwSymbol': '6L',
        'currency': 'USD',
        'varName': 'USD-BRL',
        'variableContractSize': False,
        'contractSize': 100_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '15:00',
    },
    'ty': {
        'shortName': 'ty',
        'oldName': 'ty',
        'formalName': 'US Ten Year',
        'bloombergSymbol': 'TY',
        'yellowKey': 'Comdty',
        'systemTicker': 'ty',
        'sector': 'Bonds',
        'drwSymbol': 'TY',
        'currency': 'USD',
        'varName': '10 Year Treasury Notes',
        'variableContractSize': False,
        'contractSize': 100_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '15:00',
    },
    'oat': {
        'shortName': 'oat',
        'oldName': 'oat',
        'formalName': 'French Ten Year',
        'bloombergSymbol': 'OAT',
        'yellowKey': 'Comdty',
        'systemTicker': 'oat',
        'sector': 'Bonds',
        'drwSymbol': 'FOAT',
        'currency': 'EUR',
        'varName': 'FOAT',
        'variableContractSize': False,
        'contractSize': 100_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '11:15',
    },

    #need to reorder these
    'cc': {
        'shortName': 'cc',
        'oldName': 'cocoa',
        'formalName': 'Cocoa',
        'bloombergSymbol': 'CC',
        'yellowKey': 'Comdty',
        'systemTicker': 'cc',
        'sector': 'Softs',
        'drwSymbol': 'CY',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 10,
        'bloombergDollarDivisor': 1,
        'settlementTime': '11:50',
    },
    'lc': {
        'shortName': 'lc',
        'oldName': 'livecattle',
        'formalName': 'Live Cattle',
        'bloombergSymbol': 'LC',
        'yellowKey': 'Comdty',
        'systemTicker': 'lc',
        'sector': 'Livestock',
        'drwSymbol': '48',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 40_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:00',
    },
    'qw': {
        'shortName': 'qw',
        'oldName': 'whitesugar',
        'formalName': 'White Sugar',
        'bloombergSymbol': 'QW',
        'yellowKey': 'Comdty',
        'systemTicker': 'qw',
        'sector': 'Softs',
        'drwSymbol': 'S5',
        'currency': 'USD',
        'varName': '',
        'contractSize': 50,
        'bloombergDollarDivisor': 1,
        'settlementTime': '12:55',
    },
    'df': {
        'shortName': 'df',
        'oldName': 'robusta',
        'formalName': 'Robusta',
        'bloombergSymbol': 'DF',
        'yellowKey': 'Comdty',
        'systemTicker': 'df',
        'sector': 'Softs',
        'drwSymbol': 'RC',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 50,
        'bloombergDollarDivisor': 1,
        'settlementTime': '12:25',
    },
    'or': {
        'shortName': 'or',
        'oldName': 'rubber',
        'formalName': 'Rubber',
        'bloombergSymbol': 'OR',
        'yellowKey': 'Comdty',
        'systemTicker': 'or',
        'sector': 'Softs',
        'drwSymbol': 'SGTF',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '05:00',
    },
    'mw': {
        'shortName': 'mw',
        'oldName': 'hrs',
        'formalName': 'Minneapolis Wheat',
        'bloombergSymbol': 'MW',
        'yellowKey': 'Comdty',
        'systemTicker': 'mw',
        'sector': 'Grains',
        'drwSymbol': 'MWE',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 5_000,
        'bloombergDollarDivisor': 100,
        'settlementTime': '14:15',
    },
    'es': {
        'shortName': 'es',
        'oldName': 'es',
        'formalName': 'S&P500',
        'bloombergSymbol': 'ES',
        'yellowKey': 'Index',
        'systemTicker': 'es',
        'sector': 'Stocks',
        'drwSymbol': 'RVB',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 50,
        'bloombergDollarDivisor': 1,
        'settlementTime': '15:00',
    },
    'tmt': {
        'shortName': 'tmt',
        'oldName': 'tmt',
        'formalName': 'Tetco M3',
        'bloombergSymbol': 'ZOY',
        'yellowKey': 'Comdty',
        'systemTicker': 'tmt',
        'sector': 'Energy',
        'drwSymbol': 'TMT',
        'currency': 'USD',
        'varName': '',
        'variableContractSize': False,
        'contractSize': 2500,
        'bloombergDollarDivisor': 1,
        'settlementTime': '14:30',
    },
}

commodNameOrder = [
    'crude',
    'brent',
    'diesel',
    'gasoline',
    'natgas',
    'ttf',
    'propane',
    'pjm',
    'gold',
    'silver',
    'platinum',
    'palladium',
    'copper',
    'aluminum',
    'nickel',
    'iron',
    'leanhogs',
    'feeder',
    'soymeal',
    'hrw',
    'srw',
    'corn',
    'soybean',
    'sugar',
    'cotton',
    'coffee',
    'palmoil',
    'beanoil',
    'canola',
    'euro',
    'aus',
    'yen',
    'real',
    'ty',
    'oat'
]

commodNameToFormalName = {
    'palmoil': 'Palm Oil',
    'soymeal': 'Soybean Meal',
    'sugar': 'Sugar',
    'coffee': 'Coffee',
    'crude': 'WTI',
    'brent': 'Brent',
    'natgas': 'Natural Gas',
    'gold': 'Gold',
    'platinum': 'Platinum',
    'copper': 'Copper',
    'palladium': 'Palladium',
    'cotton': 'Cotton',
    'nickel': 'Nickel',
    'aluminum': 'Aluminum',
    'leanhogs': 'Lean Hogs',
    'canola': 'Canola',
    'hrw': 'KC Wheat',
    'feeder': 'Feeder Cattle',
    'ttf': 'TTF',
    'propane': 'Propane',
    'pjm': 'PJM',
    'euro': 'EUR',
    'real': 'BRL',
    'aus': 'AUD',
    'yen': 'JPY',
    'ty': 'US Ten Year',
    'oat': 'French Ten Year',
    'tmt': 'Tetco M3',
    'iron': 'Iron Ore',
    'beanoil': 'Bean Oil',
    'diesel': 'Diesel',
    'gasoline': 'Gasoline',
    'silver': 'Silver',
    'srw': 'Chicago Wheat',
    'corn': 'Corn',
    'soybean': 'Soybeans',
}

commodNameToBloombergSym = {
    'palmoil': 'KO',
    'soymeal': 'SM',
    'sugar': 'SB',
    'coffee': 'KC',
    'crude': 'CL',
    'brent': 'CO',
    'natgas': 'NG',
    'gold': 'GC',
    'platinum': 'PL',
    'copper': 'HG',
    'nickel': 'LN',
    'aluminum': 'LA',
    'palladium': 'PA',
    'cotton': 'CT',
    'leanhogs': 'LH',
    'feeder': 'FC',
    'canola': 'RS',
    'hrw': 'KW',
    'ttf': 'TZT',
    'pjm': 'ZLS',
    'propane': 'BAP',
    'real': 'BR',
    'yen': 'JY',
    'euro': 'EC',
    'aus': 'AD',
    'ty': 'TY',
    'oat': 'OAT',
    'tmt': 'ZOY',
    'iron': 'SCO',
    'beanoil': 'BO',
    'diesel': 'HO',
    'gasoline': 'XB',
    'silver': 'SI',
    'srw': 'W ',
    'corn': 'C ',
    'soybean': 'S ',
}

commodNameToBloombergCategory = {
    'palmoil': 'Comdty',
    'soymeal': 'Comdty',
    'sugar': 'Comdty',
    'coffee': 'Comdty',
    'crude': 'Comdty',
    'brent': 'Comdty',
    'natgas': 'Comdty',
    'gold': 'Comdty',
    'platinum': 'Comdty',
    'copper': 'Comdty',
    'nickel': 'Comdty',
    'aluminum': 'Comdty',
    'palladium': 'Comdty',
    'cotton': 'Comdty',
    'leanhogs': 'Comdty',
    'feeder': 'Comdty',
    'canola': 'Comdty',
    'hrw': 'Comdty',
    'ttf': 'Comdty',
    'pjm': 'Comdty',
    'propane': 'Comdty',
    'real': 'Curncy',
    'yen': 'Curncy',
    'euro': 'Curncy',
    'aus': 'Curncy',
    'ty': 'Comdty',
    'oat': 'Comdty',
    'tmt': 'Comdty',
    'iron': 'Comdty',
    'beanoil': 'Comdty',
    'diesel': 'Comdty',
    'gasoline': 'Comdty',
    'silver': 'Comdty',
    'srw': 'Comdty',
    'corn': 'Comdty',
    'soybean': 'Comdty',
}

commodNameToTicker = {
    'palmoil': 'ko',
    'soymeal': 'sm',
    'sugar': 'sb',
    'coffee': 'kc',
    'crude': 'cl',
    'brent': 'co',
    'natgas': 'ng',
    'gold': 'gc',
    'platinum': 'pl',
    'copper': 'hg',
    'nickel': 'ln',
    'aluminum': 'la',
    'palladium': 'pa',
    'cotton': 'ct',
    'leanhogs': 'lh',
    'feeder': 'fc',
    'canola': 'rs',
    'hrw': 'kw',
    'ttf': 'ttf',
    'pjm': 'pjm',
    'propane': 'bap',
    'real': 'brl',
    'yen': 'jpy',
    'euro': 'eur',
    'aus': 'aud',
    'ty': 'ty',
    'oat': 'oat',
    'tmt': 'tmt',
    'iron': 'sco',
    'beanoil': 'bo',
    'diesel': 'ho',
    'gasoline': 'xb',
    'silver': 'si',
    'srw': 'w',
    'corn': 'c',
    'soybean': 's',
}

commodNameToSector = {
    'palmoil': 'Softs',
    'soymeal': 'Grains',
    'sugar': 'Softs',
    'coffee': 'Softs',
    'crude': 'Energy',
    'brent': 'Energy',
    'natgas': 'Energy',
    'gold': 'Metals',
    'platinum': 'Metals',
    'copper': 'Metals',
    'nickel': 'Metals',
    'aluminum': 'Metals',
    'palladium': 'Metals',
    'cotton': 'Softs',
    'leanhogs': 'Livestock',
    'feeder': 'Livestock',
    'canola': 'Softs',
    'hrw': 'Grains',
    'ttf': 'Energy',
    'pjm': 'Energy',
    'propane': 'Energy',
    'real': 'FX',
    'yen': 'FX',
    'euro': 'FX',
    'aus': 'FX',
    'ty': 'Bonds',
    'oat': 'Bonds',
    'tmt': 'Energy',
    'iron': 'Metals',
    'beanoil': 'Softs',
    'diesel': 'Energy',
    'gasoline': 'Energy',
    'silver': 'Metals',
    'srw': 'Grains',
    'corn': 'Grains',
    'soybean': 'Grains',
}

commodNameToDRWSym = {
    'palmoil': 'FCPO',
    'soymeal': 'SM',
    'sugar': 'S11',
    'coffee': 'CF',
    'crude': 'CL',
    'brent': 'BC',
    'natgas': 'NG',
    'gold': 'GC',
    'platinum': 'PL',
    'copper': 'CU',
    'nickel': 'LMENID',
    'aluminum': 'LMEAHD',
    'palladium': 'PA',
    'cotton': 'CT',
    'leanhogs': '54',
    'feeder': 'FC',
    'canola': 'W8',
    'hrw': 'KCB',
    'ttf': 'TTF',
    'pjm': 'PMI',
    'propane': 'B0',
    'real': '6L',
    'yen': 'JY',
    'euro': 'EC',
    'aus': 'AD',
    'ty': 'TY',
    'oat': 'FOAT',
    'tmt': 'TMT',
    'iron': 'SGFEF',
    'beanoil': 'BO',
    'diesel': 'HO',
    'gasoline': 'RB',
    'silver': 'SI',
    'srw': 'W',
    'corn': 'C',
    'soybean': 'S',
}

commodNameToCurrency = {
    'palmoil': 'MYR',
    'soymeal': 'USD',
    'sugar': 'USD',
    'coffee': 'USD',
    'crude': 'USD',
    'brent': 'USD',
    'natgas': 'USD',
    'gold': 'USD',
    'platinum': 'USD',
    'copper': 'USD',
    'nickel': 'USD',
    'aluminum': 'USD',
    'palladium': 'USD',
    'cotton': 'USD',
    'leanhogs': 'USD',
    'feeder': 'USD',
    'canola': 'CAD',
    'hrw': 'USD',
    'ttf': 'EUR',
    'pjm': 'USD',
    'propane': 'USD',
    'real': 'USD',
    'yen': 'USD',
    'euro': 'USD',
    'aus': 'USD',
    'ty': 'USD',
    'oat': 'EUR',
    'tmt': 'USD',
    'iron': 'USD',
    'beanoil': 'USD',
    'diesel': 'USD',
    'gasoline': 'USD',
    'silver': 'USD',
    'srw': 'USD',
    'corn': 'USD',
    'soybean': 'USD',
}

commodNameToVaRName = {
    'ty': '10 Year Treasury Notes',
    'aus': 'AUD-USD',
    'aluminum': 'Aluminum',
    'brent': 'Brent',
    'canola': 'Canola',
    'coffee': 'Coffee',
    'copper': 'Copper',
    'corn': 'Corn',
    'cotton': 'Cotton',
    'ttf': 'Dutch',
    'euro': 'EUR-USD',
    'oat': 'FOAT',
    'feeder': 'Feeder Cattle',
    'gold': 'Gold',
    'diesel': 'Heating Oil',
    'natgas': 'Henry Hub',
    'iron': 'Iron',
    'leanhogs': 'Lean Hogs',
    'nickel': 'Nickel',
    'pjm': 'PJM',
    'palladium': 'Palladium',
    'palmoil': 'Palm Oil',
    'platinum': 'Platinum',
    'propane': 'Propane',
    'gasoline': 'RBOB',
    'silver': 'Silver',
    'soymeal': 'Soybean Meal',
    'beanoil': 'Soybean Oil',
    'soybean': 'Soybeans',
    'sugar': 'Sugar',
    'real': 'USD-BRL',
    'yen': 'USD-JPY',
    'crude': 'WTI',
    'hrw': 'Wheat',
    'srw': 'Wheat',
}

commodTickerToName = reverseDict(commodNameToTicker)
commodNames = list(commodNameToTicker.keys())

weekdaysDf = pd.read_csv('C:/Users/jkarisch/Projects/common/data/weekdays.csv', parse_dates=['date'])
weekdaysSeries = weekdaysDf['date']
weekdays = weekdaysSeries.to_numpy()

contractLvd = pd.read_csv('C:/Users/jkarisch/Projects/common/data/contractLvd.csv', index_col=0).astype('datetime64')
contractFnd = pd.read_csv('C:/Users/jkarisch/Projects/common/data/contractFnd.csv', index_col=0).astype('datetime64')
contractLtd = pd.read_csv('C:/Users/jkarisch/Projects/common/data/contractLtd.csv', index_col=0).astype('datetime64')

stripContracts = pd.read_csv('C:/Users/jkarisch/Projects/common/data/stripContracts.csv', index_col=0)

variablesContractSizes = {
    'tzt': pd.read_csv('C:/Users/jkarisch/Projects/common/data/ttfContractSizes.csv'),
    'zls': pd.read_csv('C:/Users/jkarisch/Projects/common/data/pjmContractSizes.csv')
}

contractExpirationDict = {
    'fnd': contractFnd,
    'ltd': contractLtd,
    'lvd': contractLvd
}

weekdayStrToInt = {
    'monday': 0,
    'tuesday': 1,
    'wednesday': 2,
    'thursday': 3,
    'friday': 4,
    'saturday': 5,
    'sunday': 6
}
weekdayIntToStr = reverseDict(weekdayStrToInt)